/*! \file iterator_dag.hpp
 *  \brief to iterate on nodes or edges elements of a DDAG data structure.
 */
#ifndef ITERATOR_DAG_H
#define ITERATOR_DAG_H

namespace skelgis{

  //-------------------------------------------------------------------------------
  //!  Iterator nodes class
  /*!
    the iterator on nodes elements
    no template needed as this iterator is a simple rank and its incrementation
  */
  //-------------------------------------------------------------------------------
  struct iterator_dag
  //-------------------------------------------------------------------------------
  {
  public:
    unsigned int _rank;

    //! default constructor of the iterator
    //-------------------------------------------------------------------------------
    iterator_dag():_rank(0){}
    //-------------------------------------------------------------------------------
    //! constructor of the iterator
    /*!
      \param r is the initial rank to assign to the iterator
    */
    //-------------------------------------------------------------------------------
    iterator_dag(const unsigned int r):_rank(r){}
    //-------------------------------------------------------------------------------
    //! constructor of the iterator from another iterator
    /*!
      \param it is the other iterator instance from which to construct the new one
    */
    //-------------------------------------------------------------------------------
    iterator_dag(const iterator_dag& it):_rank(it._rank){}
    //-------------------------------------------------------------------------------
    //! Destructor of the iterator
    //-------------------------------------------------------------------------------
    ~iterator_dag(){}
    //-------------------------------------------------------------------------------
    //! operator ++ pre-incrementation of the iterator
    /*!
      This operation increments the position of the iterator in the DDAG on nodes.
      \return a reference to the resulting iterator itself
    */
    //-------------------------------------------------------------------------------
    inline iterator_dag& operator++(){++_rank;return *this;}
    //-------------------------------------------------------------------------------
    //! operator ++ post-incrementation of the iterator
    /*!
      This operation increments the position of the iterator in the DDAG on nodes.
      \return a reference to the resulting iterator itself
    */
    //-------------------------------------------------------------------------------
    inline iterator_dag operator++(int){iterator_dag _tmp=*this;++_rank;return _tmp;}
    //-------------------------------------------------------------------------------
    //! operator = of the iterator
    /*!
      This operation assigns another iterator informations to the current one.
      \return a reference to the resulting iterator itself
    */
    //-------------------------------------------------------------------------------
    inline iterator_dag& operator=(iterator_dag right){_rank = right._rank;return *this;}
    //-------------------------------------------------------------------------------
    //! operator = of the iterator
    /*!
      This operation assigns another iterator informations to the current one.
      \return a reference to the resulting iterator itself
    */
    //-------------------------------------------------------------------------------
    inline bool operator==(const iterator_dag& toComp) {return _rank==toComp._rank;}
    //-------------------------------------------------------------------------------
    //! operator != of the iterator
    /*!
      This operation checks the inequality of two iterators with their rank values.
      \return true if the two iterator do not have the same rank. False otherwize.
    */
    //-------------------------------------------------------------------------------
    inline bool operator!=(const iterator_dag& toComp) {return _rank!=toComp._rank;}
    //-------------------------------------------------------------------------------
    //! operator <= of the iterator
    /*!
      This operation checks if the current iterator rank is smaller or equal to the other one.
      \return true is the current iterator rank is smaller or equal. False otherwize.
    */
    //-------------------------------------------------------------------------------
    inline bool operator <=(const iterator_dag& toComp) {return _rank<=toComp._rank;}
    //-------------------------------------------------------------------------------
    //! operator <= of the iterator
    /*!
      This operation checks if the current iterator rank is biger or equal to the other one.
      \return true is the current iterator rank is biger or equal. False otherwize.
    */
    //-------------------------------------------------------------------------------
    inline bool operator <(const iterator_dag& toComp) {return _rank<toComp._rank;}
    //-------------------------------------------------------------------------------
    //! operator >= of the iterator
    /*!
      This operation checks if the current iterator rank is biger or equal to the other one.
      \return true is the current iterator rank is biger or equal. False otherwize.
    */
    //-------------------------------------------------------------------------------
    inline bool operator >=(const iterator_dag& toComp) {return _rank>=toComp._rank;}
    //-------------------------------------------------------------------------------
    //! operator > of the iterator
    /*!
      This operation checks if the current iterator rank is biger than the other one.
      \return true is the current iterator rank is biger. False otherwize.
    */
    //-------------------------------------------------------------------------------
    inline bool operator >(const iterator_dag& toComp) {return _rank>toComp._rank;}
    //-------------------------------------------------------------------------------

  };

};

#endif
